#include "InputFileTypeRegistration.hpp"

#include <registry/XmlRegistry.h>

#include <abstraction/ImmediateValueAbstraction.hpp>
#include <abstraction/XmlTokensParserAbstraction.hpp>
#include <abstraction/WrapperAbstraction.hpp>
#include <abstraction/PackingAbstraction.hpp>

namespace {

	std::shared_ptr < abstraction::OperationAbstraction > dummy ( const std::string & filename, const std::string & typehint, const ext::vector < std::string > & templateParams ) {
		if ( typehint == "set" || typehint == "Set" ) {
			ext::vector < std::shared_ptr < abstraction::OperationAbstraction > > abstractions;

			abstractions.push_back ( std::make_shared < abstraction::XmlTokensParserAbstraction > ( ) );
			abstractions.push_back ( abstraction::XmlRegistry::getXmlContainerParserAbstraction ( "Set", templateParams [ 0 ] ) );

			std::shared_ptr < abstraction::PackingAbstraction < std::string > > res = std::make_shared < abstraction::PackingAbstraction < std::string > > ( std::move ( abstractions ), 1 );
			res->setInnerConnection ( 0, 1, 0, true );
			res->setOuterConnection ( 0, 0, 0 );

			std::shared_ptr < abstraction::OperationAbstraction > tmp = res;
			std::shared_ptr < abstraction::ImmediateValueAbstraction < std::string > > file = std::make_shared < abstraction::ImmediateValueAbstraction < std::string > > ( filename );
			tmp->attachInput ( file, 0, false );
			tmp->eval ( );
			return tmp;
		} else {
			ext::vector < std::shared_ptr < abstraction::OperationAbstraction > > abstractions;

			abstractions.push_back ( std::make_shared < abstraction::XmlTokensParserAbstraction > ( ) );

			auto xmlParserAbstractionFinder = [ = ] ( const ext::deque < sax::Token > & tokens ) {
				std::string type;
				if ( typehint == "" )
					type = tokens [ 0 ].getData ( );
				else
					type = typehint;

				return abstraction::XmlRegistry::getXmlParserAbstraction ( type );
			};

			abstractions.push_back ( std::make_shared < abstraction::WrapperAbstraction < const ext::deque < sax::Token > & > > ( xmlParserAbstractionFinder ) );

			std::shared_ptr < abstraction::PackingAbstraction < std::string > > res = std::make_shared < abstraction::PackingAbstraction < std::string > > ( std::move ( abstractions ), 1 );
			res->setInnerConnection ( 0, 1, 0, true );
			res->setOuterConnection ( 0, 0, 0 );

			std::shared_ptr < abstraction::OperationAbstraction > tmp = res;
			std::shared_ptr < abstraction::ImmediateValueAbstraction < std::string > > file = std::make_shared < abstraction::ImmediateValueAbstraction < std::string > > ( filename );
			tmp->attachInput ( file, 0, false );
			tmp->eval ( );
			return tmp;
		}
	}

	std::shared_ptr < abstraction::OperationAbstraction > dummy2 ( const std::string & /*filename*/, const std::string & /*typehint*/, const ext::vector < std::string > & /*templateParams*/ ) {
		return nullptr;
	}

auto xmlInputFileHandler = registration::InputFileRegister ( "xml", dummy );
auto stringInputFileHandler = registration::InputFileRegister ( "", dummy2 );

}
